/*
OpenMorpho Library - Mathematical Morphology (Image processing) Library
Copyright (C) 2000  Laurent Najman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can contact the author at
        mailto:laurent@najman.org
and download the latest version at
        http://www.laurentnajman.org
*/

// Tree.h: interface for the CTree class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TREE_H__INCLUDED_)
#define AFX_TREE_H__INCLUDED_

#ifndef SWIG // Remove whatever is not needed by SWIG
#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <stdio.h>

#undef min
#undef max

#include <list>
#include <vector>
using namespace std;

#endif // SWIG

class CNode;
typedef list<CNode *> ListOfNodesT;

class CNode 
{
 public:
  CNode() {father = NULL; depth = Represent = number = 0;};
  CNode(int x) {father = NULL; data = x; depth = Represent = number = 0;};
  ~CNode() {sons.clear();};

  void addSon(CNode *n) {sons.push_back(n); n->father = this;};
  int data;
  CNode *father;
  ListOfNodesT sons;

  // LCA
  int depth;
  int Represent;
  int number;
};

class CTree 
{
  int nbr; // Euler nbr of nodes
  int rep;
 public:
  CTree(CNode * node);
  virtual ~CTree();

  int NbNodesGet() {return nbNodes; };
  void NbNodesSet(int n) { nbNodes=n; };
  CNode *RootGet() {return root;};
  void addSon(CNode *father, CNode *n);
  void deleteNode(CNode *node);
  void swap(CNode *father, CNode *son);

  void printTree(CNode *tr, int depth=0);
  void printTree();
  void printTree(FILE *fp);
  void printTree(FILE *fp, CNode *tr, int depth=0);
  void drawTree(FILE *fp);
  void drawTree(FILE *fp, CNode *tr, int depth=0);

  // LCA
  void LCApreprocess();
  CNode* NodesGet(int i) {return Euler[Represent[i]];};
  CNode* LCAGet(CNode *n1, CNode *n2);
  CNode* LCAGetSlow(CNode *n1, CNode *n2);
  int LCAGet(int r1, int r2);
  
  //! Test for each node of the tree that LCAGet is equal to LCAGetSlow. Return the number of differences
  int check();
  //! create a random tree of depth 'depth', each node  with at most 'n' sons on each level and return check()
  int testing(int depth, int n, CNode *father=NULL);

  //! delete all nodes
  void clearMemory();

 private:
  void clearMemory(CNode *n);  
  CNode *root;
  int nbNodes;

  // LCA
  int LCApreprocessDepthFirst(CNode *node, int depth=0);
  CNode **Euler;
  int *Represent;
  int **Minim;
  int nbRepresent;

};

#endif // !defined(AFX_TREE_H__INCLUDED_)
