#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "graphaux.h"
#include "graphes.h"

#define INFINITY LONG_MAX
#define ECHELLE 25.7
#define VITESSE 10.0

static double dist(double x1, double y1, double x2, double y2)
{
  double dx = x2 - x1;
  double dy = y2 - y1;
  return sqrt(dx * dx + dy * dy);
}

/* ====================================================================== */
/*! \fn graphe * PCCheuristique(graphe * g, int d, int a)
    \param g (entre) : un graphe pondr. La pondration de chaque arc doit 
                        se trouver dans le champ \b v_arc de la structure \b cell .
                        les coordonnes cartesiennes de chaque sommet doivent se trouver
                        dans les champs \b x et \b y de \b g.
    \param d (entre) : un sommet de \b g.
    \param a (entre) : un sommet de \b g.
    \return un plus court chemin de d vers a sous forme d'un graphe. 
    \brief trouve un plus court chemin de d vers a, et retourne ce chemin sous forme d'un graphe. 
*/
graphe * PCCheuristique(graphe * g, int d, int a)
/* ====================================================================== */
{
  int n = g->nsom;
  boolean * S = EnsembleVide(n);
  int k, r, s;
  pcell p;
  TYP_VARC vmin, v, h;
  graphe * pcc = InitGraphe(n, n-1); /* pour le resultat */
  graphe * g_1 = Symetrique(g);
  int n_som_explore = 0;
  int *T = ListeVide(n);
  int ts, tr, tt = 0;

  // DIJKSTRA AVEC HEURISTIQUE

  S[d] = TRUE;
  for (k = 0; k < n; k++) g->v_sommets[k] = INFINITY;
  g->v_sommets[d] = 0;
  s = d;
  do 
  {
    n_som_explore++;
    pcc->v_sommets[s] = 1; // pour le coloriage des sommets explores
    for (p = g->gamma[s]; p != NULL; p = p->next)
    { /* pour tout r successeur de s */
      r = p->som;
      if (!S[r])
      {
        v = g->v_sommets[s]+p->v_arc;
        if (g->v_sommets[r] > v) g->v_sommets[r] = v;
        T[tt] = r; // insere r dans T
        tt++;
      } // if (!S[r])
    } // for p
    // extraire un sommet s de T de valeur g->v_sommets+h(s) minimale
    // avec h(s) = dist(s,a)*ECHELLE/VITESSE

    ts = 0; 
    s = T[0];
    h = (TYP_VARC)(dist(g->x[s],g->y[s],g->x[a],g->y[a]) * ECHELLE / VITESSE);
    vmin = g->v_sommets[s] + h;
    for (tr = 1; tr < tt; tr++)
    {
      r = T[tr];
      h = (TYP_VARC)(dist(g->x[r],g->y[r],g->x[a],g->y[a]) * ECHELLE / VITESSE);
      if (g->v_sommets[r]+h <= vmin) { s = r; ts = tr; vmin = g->v_sommets[r]+h; }
    }
    S[s] = TRUE;
    for (; ts < tt-1; ts++) T[ts] = T[ts+1]; // retire s de T
    tt--;
  } while ((s != a) && (tt > 0));
  if (s != a)
  {
    printf("WARNING: pas de pcc trouve \n");
    return pcc;    
  }
  printf("pi(a) = %d\n", g->v_sommets[a]);
  printf("nb. sommets explores = %d\n", n_som_explore);

  // CONSTRUCTION DU PCC

  while (s != d)
  {
    for (p = g_1->gamma[s]; p != NULL; p = p->next)
    { /* pour tout r predecesseur de s */
      r = p->som;
      if ((g->v_sommets[s]-g->v_sommets[r]) == p->v_arc) 
      {
        AjouteArcValue(pcc, r, s, p->v_arc);
        s = r;
        break;
      }
    } // for p
    if (p == NULL)
    {
      printf("WARNING: pas de pcc trouve \n");
      TermineGraphe(g_1);
      return pcc;
    }
  }
  TermineGraphe(g_1);
  for (k = 0; k < n; k++) { pcc->x[k] = g->x[k]; pcc->y[k] = g->y[k]; }
  return pcc;

} /* PCCheuristique() */

/* ====================================================================== */
int main(int argc, char **argv)
/* ====================================================================== */
{
  graphe *g, *pcc;
  int na, ns;     /* nombre d'arcs, nombre de sommets */
  int d, a;

  if (argc != 4)
  {
    fprintf(stderr, "usage: %s graphe_value d a\n", argv[0]);
    exit(0);
  }

  g = ReadGraphe(argv[1]);             /* lit le graphe a partir du fichier */
  d = atoi(argv[2]);
  a = atoi(argv[3]);
  pcc = PCCheuristique(g, d, a);
  
  // genere une figure en PostScript
  EPSGraphe(pcc, "pcc.ps", 4, 2, 25, 0, 0, 1, 0);

  TermineGraphe(g);
  TermineGraphe(pcc);

  return 0;
} /* main() */
