#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "graphaux.h"
#include "graphes.h"

double DistanceEuclidienne(double x1, double y1, double x2, double y2)
{
  double dx = x2 - x1;
  double dy = y2 - y1;
  return sqrt(dx * dx + dy * dy);
}

int main(int argc, char **argv)
{
  graphe *g, *g_1, *a;
  int na, ns;     /* nombre d'arcs, nombre de sommets */
  chrono Chrono1; /* pour la mesure du temps */
  int temps;      /* idem */
  FILE *fd;
  int i, j, narc;

  if (argc != 2)
  {
    fprintf(stderr, "usage: %s <listepoints>\n", argv[0]);
    exit(0);
  }
  
  fd = fopen(argv[1], "r");
  if (!fd)
  {
    fprintf(stderr, "cannot open file: %s\n", argv[1]);
    exit(0);
  }
  fscanf(fd, "%d", &ns);
  na = (ns * (ns - 1)) / 2;
  g = InitGraphe(ns, na);
  for (i = 0; i < ns; i++)
    fscanf(fd, "%lf %lf", &(g->x[i]), &(g->y[i]));
  fclose(fd);

  // construit le graphe des distances sur les sommets de g (antisymetrique et sans boucle)
  narc = 0;
  for (i = 0; i < ns; i++)
    for (j = i+1; j < ns; j++)
    {
      AjouteArc(g, i, j);
      g->tete[narc] = i;
      g->queue[narc] = j;
      g->v_arcs[narc] = -(long)DistanceEuclidienne(g->x[i], g->y[i], g->x[j], g->y[j]);
      narc++;
    }

  // calcule l'arbre de poids maximum sur g
  g_1 = Symetrique(g);
  a = Kruskal1(g, g_1);

  // genere une figure en PostScript
  PSGraphe(a, "graph.ps", 4, 0, 5);

  TermineGraphe(g);
  TermineGraphe(g_1);
  TermineGraphe(a);

  return 0;
} /* main() */
