
/* ====================================================================== */
/*! \fn graphe * InitGrapheOpt(int nsom, int nmaxarc, int al_arcs, int al_varc, int al_vsom, int al_coord)
    \param   nsom (entre) : nombre de sommets.
    \param nmaxarc (entre) : nombre maximum d'arcs.
    \param al_arcs (entre) : boolen indiquant s'il faut allouer la mmoire pour les listes d'arcs.
    \param al_varc (entre) : boolen indiquant s'il faut allouer la mmoire pour les valeurs d'arcs.
    \param al_vsom (entre) : boolen indiquant s'il faut allouer la mmoire pour les valeurs de sommets.
    \param al_coord (entre) : boolen indiquant s'il faut allouer la mmoire pour les coord. des sommets.
    \return un graphe.
    \brief alloue la memoire minimale pour reprsenter un graphe a 'nsom' sommets,
              possdant au maximum 'nmaxarc' arcs. 
              Retourne un pointeur sur la structure alloue. 
*/
graphe * InitGrapheOpt(int nsom, int nmaxarc, int al_arcs, int al_varc, int al_vsom, int al_coord)
/* ====================================================================== */
{
  graphe * g;
  int i;
  
  g = (graphe *)malloc(sizeof(graphe));
  if (g == NULL)
  {   fprintf(stderr, "InitGraphe : malloc failed\n");
      exit(0);
  }

  g->reserve = (cell *)malloc(nmaxarc * sizeof(cell));
  if (g->reserve == NULL)
  {   fprintf(stderr, "InitGraphe : malloc failed\n");
      exit(0);
  }

  g->gamma = (pcell *)calloc(nsom, sizeof(pcell)); /* calloc met a 0 la memoire allouee */
  if (g->gamma == NULL)
  {   fprintf(stderr, "InitGraphe : calloc failed\n");
      exit(0);
  }

  if (al_arcs)
  {
    g->tete = (int *)malloc(nmaxarc * sizeof(int));
    g->queue = (int *)malloc(nmaxarc * sizeof(int));
    if ((g->tete == NULL) || (g->tete == NULL))
    {   fprintf(stderr, "InitGraphe : malloc failed\n");
        exit(0);
    }
  } else g->tete = g->queue = NULL;

  if (al_varc)
  {
    g->v_arcs = (double *)malloc(nmaxarc * sizeof(double));
    if (g->v_arcs == NULL)
    {   fprintf(stderr, "InitGraphe : malloc failed\n");
        exit(0);
    }
  } else g->v_arcs = NULL;

  if (al_vsom)
  {
    g->v_sommets = (double *)malloc(nsom * sizeof(double));
    if (g->v_sommets == NULL)
    {   fprintf(stderr, "InitGraphe : malloc failed\n");
        exit(0);
    }
  } else g->v_sommets = NULL;

  if (al_coord)
  {
    g->x = (double *)malloc(nsom * sizeof(double));
    g->y = (double *)malloc(nsom * sizeof(double));
    if ((g->x == NULL) || (g->y == NULL))
    {   fprintf(stderr, "InitGraphe : malloc failed\n");
        exit(0);
    }
  } else g->x = g->y = NULL;

  g->nsom = nsom;
  g->nmaxarc = nmaxarc;
  g->narc = 0;

  /* construit la liste initiale de cellules libres */
  for (i = 0; i < nmaxarc - 1; i++)
    (g->reserve+i)->next = g->reserve+i+1;
  (g->reserve+i)->next = NULL;
  g->libre = g->reserve;  

  g->nomsommet = NULL;
  
  return g;
} /* InitGrapheOpt() */

/* ====================================================================== */
/*! \fn graphe * ReadGrapheOpt(char * filename, int al_arcs, int al_varc, int al_vsom, int al_coord)
    \param   filename (entre) : nom du fichier graphe.
    \param al_arcs (entre) : boolen indiquant s'il faut allouer la mmoire pour les listes d'arcs.
    \param al_varc (entre) : boolen indiquant s'il faut allouer la mmoire pour les valeurs d'arcs.
    \param al_vsom (entre) : boolen indiquant s'il faut allouer la mmoire pour les valeurs de sommets.
    \param al_coord (entre) : boolen indiquant s'il faut allouer la mmoire pour les coord. des sommets.
    \return un graphe.
    \brief Lit les donnes d'un graphe dans le fichier filename, retourne un pointeur sur la structure graphe construite. 
*/
graphe * ReadGrapheOpt(char * filename, int al_arcs, int al_varc, int al_vsom, int al_coord)
/* ====================================================================== */
{
  graphe * g;
  int i, n, m, t, q;
  char buf[256];
  char *ret;

  FILE *fd = NULL;

  fd = fopen(filename,"r");
  if (!fd)
  {
    fprintf(stderr, "ReadGraphe: file not found: %s\n", filename);
    return NULL;
  }

  fscanf(fd, "%d %d\n", &n, &m);
  g = InitGrapheOpt(n, m, al_arcs, al_varc, al_vsom, al_coord);
  do
  {
    ret = fgets(buf, 256, fd);
    if ((ret != NULL) && (strncmp(buf, "noms sommets", 12) == 0))
    {
      g->nomsommet = (char **)malloc(n * sizeof(char *));
      if (g->nomsommet == NULL)
      {   fprintf(stderr, "ReadGraphe : malloc failed\n");
          exit(0);
      }
      for (i = 0; i < n; i++)
      {
        fscanf(fd, "%d %s\n", &t, buf);
        g->nomsommet[t] = (char *)malloc((strlen(buf)+1) * sizeof(char));
        if (g->nomsommet[t] == NULL)
        {   fprintf(stderr, "ReadGraphe : malloc failed\n");
            exit(0);
        }
        strcpy(g->nomsommet[t], buf);
      } /* for (i = 0; i < n; i++) */
    } /* if ((ret != NULL) && (strcmp(buf, "noms sommets") == 0)) */
    else if ((ret != NULL) && (strncmp(buf, "arcs", 4) == 0))
    {
      for (i = 0; i < m; i++)  
      {
        fscanf(fd, "%d %d\n", &t, &q);
        AjouteArc(g, t, q);
      }
    } /*  if ((ret != NULL) && (strncmp(buf, "arcs", 4) == 0)) */
  } while (ret != NULL);

  return g;
} /* ReadGrapheOpt() */
