
/*************************************************
   Fonctions pour la gestion d'une liste ordonnee
   Michel Couprie 2002
**************************************************/

/* ==================================== */
/*! \fn Listo * CreeListoVide(int taillemax)
    \param taillemax (entre) : taille maxi de la liste.
    \return une liste vide.
    \brief alloue une liste vide.
*/
Listo * CreeListoVide(int taillemax)
/* ==================================== */
{
  Listo * L = (Listo *)malloc(sizeof(Listo) + sizeof(int) * (taillemax-1));
  if (L == NULL)
  {   fprintf(stderr, "CreeListoVide : malloc failed\n");
      exit(0);
  }  
  L->Max = taillemax;
  L->Sp = 0;
  return L;
} // CreeListoVide()

/* ==================================== */
/*! \fn void ListoFlush(Listo * L)
    \param L (entre) : une liste ordonne.
    \brief r-initialise L (tous les lments prsents sont retirs).
*/
void ListoFlush(Listo * L)
/* ==================================== */
{
  L->Sp = 0;
} // ListoFlush()

/* ==================================== */
/*! \fn boolean ListoVide(Listo * L)
    \param L (entre) : une liste Listo.
    \return TRUE si la liste \b L est vide, FALSE sinon.
    \brief teste si la liste \b est vide.
*/
boolean ListoVide(Listo * L)
/* ==================================== */
{
  return (boolean)(L->Sp == 0);
} // ListoVide()

/* ==================================== */
/*! \fn int ListoPopmin(Listo * L)
    \param L (entre) : une liste ordonne.
    \return la valeur se trouvant en dbut de liste.
    \brief retire de la liste \b L la valeur se trouvant au dbut, et retourne cette valeur.
    \warning si la liste \b L est vide, la fonction systme \b exit est appele.
*/
int ListoPopmin(Listo * L)
/* ==================================== */
{
  if (L->Sp == 0)
  {
    fprintf(stderr, "erreur Listo vide\n");
    exit(1);
  }
  L->Sp -= 1;
  return L->Pts[L->Sp];
} // ListoPopmin()
  
/* ==================================== */
/*! \fn void ListoInsert(Listo * L, int V)
    \param L (entre) : une liste ordonne.
    \brief insre la valeur \b V dans la liste ordonne \b L .
    \warning si la liste \b L est pleine, la fonction systme \b exit est appele.
*/
void ListoInsert(Listo * L, int V)
/* ==================================== */
{
  int tmp, i;
  if (L->Sp > L->Max - 1)
  {
    fprintf(stderr, "erreur Listo pleine\n");
    exit(1);
  }
  L->Pts[L->Sp] = V;
  i = L->Sp;
  L->Sp += 1;
  while ((i > 0) && (L->Pts[i] < L->Pts[i-1]))
  {
    tmp = L->Pts[i];
    L->Pts[i] = L->Pts[i-1];
    L->Pts[i-1] = tmp;
    i--;
  }
} // ListoInsert()

/* ==================================== */
/*! \fn void ListoPrint(Listo * L)
    \param L (entre) : une liste ordonne.
    \brief affiche le contenu de la liste \b L.
*/
void ListoPrint(Listo * L)
/* ==================================== */
{
  int i;
  if (ListoVide(L)) {printf("[]"); return;}
  printf("[ ");
  for (i = 0; i < L->Sp; i++)
    printf("%d ", L->Pts[i]);
  printf("]");
} // ListoPrint()

/* ==================================== */
/*! \fn void ListoPrintLine(Listo * L)
    \param L (entre) : une liste ordonne.
    \brief affiche le contenu de la liste \b L, suivi d'un retour ligne.
*/
void ListoPrintLine(Listo * L)
/* ==================================== */
{
  int i;
  if (ListoVide(L)) {printf("[]\n"); return;}
/*
  printf("Max = %d ; Sp = %d \n", L->Max, L->Sp);
*/
  printf("[ ");
  for (i = 0; i < L->Sp; i++)
    printf("%d ", L->Pts[i]);
  printf("]\n");
} // ListoPrintLine()

/* ==================================== */
/*! \fn void ListoTermine(Listo * L)
    \param L (entre) : une liste ordonne.
    \brief rcupre l'espace mmoire occup par la liste \b L.
*/
void ListoTermine(Listo * L)
/* ==================================== */
{
  free(L);
} // ListoTermine() 
