#include <stdlib.h>
#include <stdio.h>
#include "graphaux.h"
#include "graphes.h"

#define USAGE "affiche la composante connexe associee au sommet (parcours en largeur)"

/* ====================================================================== */
graphe * symetrique(graphe * g)
/* ====================================================================== */
{
  graphe *g_1;
  int nsom, narc, al_arcs, k, i, j;
  pcell p;

  nsom = g->nsom;
  narc = g->narc;
  g_1 = InitGraphe(nsom, narc);

  for (i = 0; i < nsom; i++) /* pour tout i sommet de g */
  {
    p = g->gamma[i]; /* acces a la liste des successeurs de i */
    for ( ; p != NULL; p = p->next) /* pour tout j successeur de i */
    {
      j = p->som;
      AjouteArc(g_1, j, i);
    }
  }

  return g_1;
} /* symetrique() */

/* ====================================================================== */
/*! \fn void ParcoursLargeur(graphe * g, graphe *g_1, int a)
    \param   g (entre) : un graphe.
    \param g_1 (entre) : le graphe symtrique de g.
    \param a (entre) : un sommet du graphe g.
    \brief parcourt en largeur d'abord la composante connexe associe au sommet \b a, 
           et imprime les sommets des diffrents niveaux
*/
void ParcoursLargeur(graphe * g, graphe *g_1, int a)
/* ====================================================================== */
{
  Lifo * T1;   /* liste temporaire geree en pile (Last In, First Out) */
  Lifo * T2;   /* liste temporaire geree en pile (Last In, First Out) */
  Lifo * T;    /* pour l'echange */
  int i, n, s, niv;
  pcell p;
  boolean * Ca;  /* pour eviter de reparcourir un sommet plusieurs fois */

  n = g->nsom;
  Ca = EnsembleVide(n);
  T1 = CreeLifoVide(n);
  T2 = CreeLifoVide(n);

  niv = 0;
  Ca[a] = TRUE;
  LifoPush(T1, a);  
  while (!LifoVide(T1)) /* boucle jusqu'a stabilite */
  {
    printf("niveau %d : ", niv);
    while (!LifoVide(T1)) /* boucle pour le parcours d'un niveau */
    {
      i = LifoPop(T1);
      printf("%d ", i);
      p = g->gamma[i]; /* acces a la liste des successeurs de i */
      for ( ; p != NULL; p = p->next) /* pour tout s successeur de i */
      {
        s = p->som;
        if (!Ca[s]) 
        {
          Ca[s] = TRUE;
          LifoPush(T2, s);
        }
      }
      p = g_1->gamma[i]; /* acces a la liste des successeurs de i dans g_1*/
      for ( ; p != NULL; p = p->next) /* pour tout s successeur de i */
      {
        s = p->som;
        if (!Ca[s]) 
        {
          Ca[s] = TRUE;
          LifoPush(T2, s);
        }
      }
    } // fin du niveau
    T = T2;
    T2 = T1;
    T1 = T;
    niv++;
    printf("\n");
  }

  LifoTermine(T1);
  LifoTermine(T2);
  free(Ca);
} /* ParcoursLargeur() */

/* ====================================================================== */
int main(int argc, char **argv)
/* ====================================================================== */
{
  graphe *g, *g_1;
  int x;

  if (argc != 3)
  {
    fprintf(stderr, "usage: %s <nom fichier> <numero sommet>\n%s\n", argv[0], USAGE);
    exit(0);
  }

  g = ReadGraphe(argv[1]); /* lit le graphe a partir du fichier */
  g_1 = symetrique(g);            /* calcule le symetrique de g */
  x = atoi(argv[2]);
  ParcoursLargeur(g, g_1, x);

  TermineGraphe(g);
  TermineGraphe(g_1);

  return 0;
} /* main() */
