#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "graphaux.h"
#include "graphes.h"

#define INFINITY LONG_MAX

/* ====================================================================== */
/*! \fn graphe * PCC_(graphe * g, int i, int a)
    \param g (entre) : un graphe pondr. La pondration de chaque arc doit 
                        se trouver dans le champ \b v_arc de la structure \b cell .
    \param i (entre) : un sommet de \b g.
    \param a (entre) : un sommet de \b g.
    \return un plus court chemin de d vers a sous forme d'un graphe. 
    \brief trouve un plus court chemin de d vers a, et retourne ce chemin sous forme d'un graphe. 
*/
graphe *PCC_(graphe * g, int i, int a)
/* ====================================================================== */
{
  int n = g->nsom;
  boolean * S = EnsembleVide(n);
  int k, s, t;
  pcell p;
  TYP_VARC vmin, v;
  graphe * pcc = InitGraphe(n, n-1); /* pour le resultat */
  graphe * g_1 = Symetrique(g);
  int n_som_explore = 0;

  // DIJKSTRA (avec arret des que le sommet a est atteint)

  S[i] = TRUE;
  for (k = 0; k < n; k++) g->v_sommets[k] = INFINITY;
  g->v_sommets[i] = 0;
  s = i;
  while ((s != a) && (g->v_sommets[s] < INFINITY))
  {
    n_som_explore++;
    pcc->v_sommets[s] = 1; // pour le coloriage des sommets explores
    for (p = g->gamma[s]; p != NULL; p = p->next)
    { /* pour tout t successeur de s */
      t = p->som;
      if (!S[t])
      {
        v = g->v_sommets[s]+p->v_arc;
        if (g->v_sommets[t] > v) g->v_sommets[t] = v;
      } // if (!S[t])
    } // for p
    // extraire un sommet s hors de S de valeur g->v_sommets minimale
    vmin = INFINITY;
    for (t = 0; t < n; t++)
      if (!S[t]) 
        if (g->v_sommets[t] <= vmin) { s = t; vmin = g->v_sommets[t]; }
    S[s] = TRUE;
  } // while ((s != a) && (g->v_sommets[s] < INFINITY))
  if (s != a)
  {
    printf("WARNING: pas de pcc trouve (s = %d , a = %d)\n", s, a);
    return NULL;    
  }
  printf("pi(a) = %d\n", g->v_sommets[a]);
  printf("nb. sommets explores = %d\n", n_som_explore);

  // CONSTRUCTION DU PCC

  while (s != i)
  {
    for (p = g_1->gamma[s]; p != NULL; p = p->next)
    { /* pour tout t predecesseur de s */
      t = p->som;
      if ((g->v_sommets[s]-g->v_sommets[t]) == p->v_arc) 
      {
        AjouteArcValue(pcc, t, s, p->v_arc);
        s = t;
        break;
      }
    } // for p
  }
  TermineGraphe(g_1);
  for (k = 0; k < n; k++) { pcc->x[k] = g->x[k]; pcc->y[k] = g->y[k]; }
  return pcc;

} /* PCC_() */

/* ====================================================================== */
int main(int argc, char **argv)
/* ====================================================================== */
{
  graphe *g, *pcc;
  int na, ns;     /* nombre d'arcs, nombre de sommets */
  int d, a;

  if (argc != 4)
  {
    fprintf(stderr, "usage: %s graphe_value d a\n", argv[0]);
    exit(0);
  }

  g = ReadGraphe(argv[1]);             /* lit le graphe a partir du fichier */
  d = atoi(argv[2]);
  a = atoi(argv[3]);
  pcc = PCC_(g, d, a);
  
  // genere une figure en PostScript
  EPSGraphe(pcc, "pcc.ps", 4, 2, 25, 0, 0, 1, 0);

  TermineGraphe(g);
  TermineGraphe(pcc);

  return 0;
} /* main() */
